{{      IR_Remote_NewCog.spin
      Original code by Tom Doyle
      2 March 2007                
      modified by Dave Ratcliff--May 18 2009 to continuously run in new cog   }}
  NoNewCode    =    255    
  gapMin       =   2000             ' minimum idle gap - adjust to eliminate auto repeat
  startBitMin  =   2000             ' minimum length of start bit in us (2400 us reference)
  startBitMax  =   2800             ' maximum length of start bit in us (2400 us reference)
  oneBitMin    =   1000             ' minimum length of 1 (1200 us reference)
  oneBitMax    =   1400             ' maximum length of 1 (1200 us reference)

VAR

  byte  cog
  long  Stack[20]  

PUB Start(Pin, addrMainCode,addrmaindev) : result
{{
   Pin - propeller pin connected to IR receiver
   addrMainCode - address of keycode variable in main memory
}}

    stop
    byte[addrMainCode] := NoNewCode
    byte[addrmaindev] := nonewcode
    cog := cognew(getSonycode(Pin, addrMainCode,addrmaindev), @Stack) + 1
    result := cog

PUB Stop
{{
   stop cog if in use
}}

    if cog
      cogstop(cog~ -1)
    
PUB getSonyCode(pin, addrMainCode,addrmaindev) | irCode, irdev, index, pulseWidth, lockID
{{
   Decode the Sony TV Remote key code from pulses received by the IR receiver
}}

   ' wait for idle period (ir receiver output = 1 for gapMin)
   ' comment out "auto repeat" code if auto key repeat is desired
   
   ' start of "auto repeat" code section
   dira[pin]~
   index := 0
 repeat {{DR - added this extra repeat to not let cog die and keep updating addrmaincode}} 
   repeat
     if ina[Pin] == 1
       index++
     else
       index := 0
   while index < gapMin
   ' end of "auto repeat" code section
   
   byte[addrMainCode] := 255 {{DR - added this to 0 out addrmaincode when no buttons are pressed}}
   byte[addrmaindev] := 255 {{DR - added this to change addrmain when no buttons are pressed}}
   frqa := 1
   ctra := 0
   dira[pin]~
   
   ' wait for a start pulse ( width > startBitMin and < startBitMax  )
   
   repeat      
      ctra := (%10101 << 26 ) | (PIN)                      ' accumulate while A = 0  
      waitpne(0 << pin, |< Pin, 0)                         
      phsa:=0                                              ' zero width
      waitpeq(0 << pin, |< Pin, 0)                         ' start counting
      waitpne(0 << pin, |< Pin, 0)                         ' stop counting                                               
      pulseWidth := phsa  / (clkfreq / 1_000_000) + 1    
   while ((pulseWidth < startBitMin) OR (pulseWidth > startBitMax))

   ' read in next 12 bits
   index := 0
   irCode := 0
   repeat
      ctra := (%10101 << 26 ) | (PIN)                      ' accumulate while A = 0  
      waitpne(0 << pin, |< Pin, 0)                         
      phsa:=0                                              ' zero width
      waitpeq(0 << pin, |< Pin, 0)                         ' start counting
      waitpne(0 << pin, |< Pin, 0)                         ' stop counting                                               
      pulseWidth := phsa  / (clkfreq / 1_000_000) + 1
      
    if (pulseWidth > oneBitMin) AND (pulseWidth < oneBitMax)
       irCode := irCode + (1 << index)
    index++
   while index < 11

   irDev  := (irCode & $f80) >> 7  ''Masking of upper 5 bits           
   irCode := irCode & $7f   ''Masking of lower 7 bits                  
                                                                       
                                                                       
   byte[addrMainCode] := irCode                                        
   byte[addrMainDev]  := irDev   ''byte[addrMainDev] = upper 5 bits    






   
   